from millet.skill import SkillResult, Skill


def test_eq():
    class EmptySkill(Skill):
        def start(self, initial_message: str):
            pass

    skill1 = EmptySkill()
    skill2 = EmptySkill()

    assert skill1 == skill2


def test_skill_result_repr():
    skill_result = SkillResult(answers=["Hi"], relevant=True)
    assert repr(skill_result) == "SkillResult(answers=['Hi'], relevant=True)"


def test_say():
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.say(initial_message)

    skill = EchoSkill()
    assert skill.send("Hi") == SkillResult(answers=["Hi"], relevant=True) and skill.finished


def test_duplicate_message_say():
    class DuplicateEchoSkill(Skill):
        def start(self, initial_message: str):
            self.say(initial_message * 2)

    skill = DuplicateEchoSkill()
    assert skill.send("Hi") == SkillResult(answers=["HiHi"], relevant=True) and skill.finished


def test_multi_answers():
    class GreetingSkill(Skill):
        def start(self, initial_message: str):
            self.say("Hello")
            self.say("How are you?")

    skill = GreetingSkill()
    assert skill.send("Hi") == SkillResult(answers=["Hello", "How are you?"], relevant=True) and skill.finished


def test_stop_exception():
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.say(initial_message)

    skill = EchoSkill()
    skill.send("Hi")
    try:
        skill.send("Hi")
        assert False, f"Expected {StopIteration.__name__}"
    except StopIteration:
        assert True


def test_ask():
    class AgeSkill(Skill):
        def start(self, initial_message: str):
            age = self.ask("How old are you?")
            self.say(f"You are {age} years old!")

    skill = AgeSkill()
    assert skill.send("Hi") == SkillResult(answers=["How old are you?"], relevant=True) and not skill.finished
    assert skill.send("42") == SkillResult(answers=["You are 42 years old!"], relevant=True) and skill.finished


def test_ask_with_direct_to():
    class AgeSkill(Skill):
        def start(self, initial_message: str):
            self.ask("How old are you?", direct_to=self.wait_age)

        def wait_age(self, age: str):
            self.say(f"You are {age} years old!")

    skill = AgeSkill()
    assert skill.send("Hi") == SkillResult(answers=["How old are you?"], relevant=True) and not skill.finished
    assert skill.send("42") == SkillResult(answers=["You are 42 years old!"], relevant=True) and skill.finished


def test_specify():
    class AgeSkill(Skill):
        def start(self, initial_message: str):
            age = self.ask("How old are you?")
            try:
                age = int(age)
            except:
                age = self.specify("Incorrect age. Repeat pls.")
            self.say(f"You are {age} years old!")

    skill = AgeSkill()
    assert skill.send("Hi") == SkillResult(answers=["How old are you?"], relevant=True) and not skill.finished
    assert skill.send("I don't know!") == SkillResult(answers=["Incorrect age. Repeat pls."], relevant=False) and not skill.finished
    assert skill.send("42") == SkillResult(answers=["You are 42 years old!"], relevant=True) and skill.finished


def test_specify_with_direct_to():
    class AgeSkill(Skill):
        def start(self, initial_message: str):
            self.ask("How old are you?", direct_to=self.waiting_age)

        def waiting_age(self, age: str):
            try:
                age = int(age)
            except:
                age = self.specify("Incorrect age. Repeat pls.")
            self.say(f"You are {age} years old!")

    skill = AgeSkill()
    assert skill.send("Hi") == SkillResult(answers=["How old are you?"], relevant=True) and not skill.finished
    assert skill.send("I don't know!") == SkillResult(answers=["Incorrect age. Repeat pls."], relevant=False) and not skill.finished
    assert skill.send("42") == SkillResult(answers=["You are 42 years old!"], relevant=True) and skill.finished


def test_persistent_initial_message():
    class FriendNameSkill(Skill):
        def start(self, initial_message: str):
            self.say(f"Nice to meet you {initial_message}!")
            friend_name = self.ask("What's your friend's name?")
            self.say(f"Your friend is {friend_name}!")

    skill = FriendNameSkill()
    assert skill.send("Bob") == SkillResult(answers=["Nice to meet you Bob!", "What's your friend's name?"], relevant=True) and not skill.finished
    assert skill.send("Alice") == SkillResult(answers=["Your friend is Alice!"], relevant=True) and skill.finished


def test_reset():
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.say(initial_message)

    skill = EchoSkill()
    assert skill.send("Hi") == SkillResult(answers=["Hi"], relevant=True) and skill.finished

    skill.reset()
    assert skill.send("Hi") == SkillResult(answers=["Hi"], relevant=True) and skill.finished


def test_restart():
    class RestartSkill(Skill):
        def start(self, initial_message: str):
            self.say('Start')
            self.ask('Continue?', direct_to=self.waiting_answer)

        def waiting_answer(self, answer: str):
            if answer == 'Yes':
                self.restart(initial_message=answer)
            else:
                self.say('Bye')

    skill = RestartSkill()
    assert skill.send("Hi") == SkillResult(answers=["Start", "Continue?"], relevant=True) and not skill.finished
    assert skill.send("Yes") == SkillResult(answers=["Start", "Continue?"], relevant=True) and not skill.finished
    assert skill.send("No") == SkillResult(answers=["Bye"], relevant=True) and skill.finished


def test_retry():
    class RestartSkill(Skill):
        def start(self, initial_message: str):
            self.say('Start')
            self.ask('Continue?', direct_to=self.waiting_answer)

        def waiting_answer(self, answer: str):
            if answer == 'Yes':
                self.restart(initial_message=answer)
            elif answer == "No":
                self.say('Bye')
            else:
                self.retry(initial_message=answer)

    skill = RestartSkill()
    assert skill.send("Hi") == SkillResult(answers=["Start", "Continue?"], relevant=True) and not skill.finished
    assert skill.send("I don't know") == SkillResult(answers=["Start", "Continue?"], relevant=False) and not skill.finished
    assert skill.send("No") == SkillResult(answers=["Bye"], relevant=True) and skill.finished


def test_finish():
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.foo(initial_message)

        def foo(self, message: str):
            self.finish(message)

    skill = EchoSkill()
    assert skill.send("Hi") == SkillResult(answers=["Hi"], relevant=True) and skill.finished


def test_abort():
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.foo(initial_message)

        def foo(self, message: str):
            self.abort(message)

    skill = EchoSkill()
    assert skill.send("Hi") == SkillResult(answers=["Hi"], relevant=False) and skill.finished


def test_number_of_starts():
    class AgeSkill(Skill):
        def __init__(self):
            self.number_of_starts = 0
            super().__init__()

        def start(self, initial_message: str):
            self.number_of_starts += 1
            age = self.ask("How old are you?")
            self.say(f"You are {age} years old!")

    skill = AgeSkill()
    skill.send("Hi")
    assert skill.number_of_starts == 1
    skill.send("42")
    assert skill.number_of_starts == 2



from typing import List, Type

import pytest

from redis import Redis

from millet import Skill, Agent
from millet.context import UserContext


@pytest.fixture
def bob_id():
    return "Bob"


@pytest.fixture
def alice_id():
    return "Alice"


@pytest.fixture
def meeting_skill_class() -> Type[Skill]:
    class MeetingSkill(Skill):
        def start(self, initial_message: str):
            self.ask(question="What is your name?", direct_to=self.waiting_name)

        def waiting_name(self, name: str):
            self.say(f"Nice to meet you {name}!")

    return MeetingSkill


@pytest.fixture
def meeting_agent(meeting_skill_class: Type[Skill]) -> Agent:
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "Hello" in message:
            skills.append(meeting_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    return agent


# pickle serializer requires global class for serialize/deserialize
class AgeSkill(Skill):
    def start(self, initial_message: str):
        self.ask("How old are you?", direct_to=self.waiting_age)

    def waiting_age(self, age: str):
        try:
            age = int(age)
        except:
            self.specify("Incorrect age: expected number, repeat pls", direct_to=self.waiting_age)
        self.say("Ok")


@pytest.fixture
def age_skill_class():
    return AgeSkill


@pytest.fixture
def redis():
    redis = Redis()
    redis.flushdb()
    yield redis
    redis.flushdb()


@pytest.fixture
def user_context():
    global_params = {
        "name": "Bob",
        "age": 42,
    }

    return UserContext(params=global_params, skills=[AgeSkill()])



from abc import ABC, abstractmethod
from typing import List, Optional, Callable


class SkillSignal(Exception):
    pass


class IterationSignal(SkillSignal):
    def __init__(self, *, relevant: bool):
        self.relevant = relevant


class RestartIterationSignal(IterationSignal):
    pass


class FinishIterationSignal(IterationSignal):
    pass


class SkillResult:
    def __init__(self, *, answers: List[str], relevant: bool):
        self.answers = answers
        self.relevant = relevant

    def __repr__(self):
        return f"{self.__class__.__name__}(answers={self.answers}, relevant={self.relevant})"

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.answers == other.answers and self.relevant == other.relevant


class Skill(ABC):
    def __init__(self):
        self._finished = False
        self._context = {}
        self._expected_question_key = None
        self._current_state = self.start
        self._initial_message = None
        self.global_context = None
        self._answers = []

    def reset(self) -> None:
        self._finished = False
        self._context = {}
        self._expected_question_key = None
        self._current_state = self.start
        self._initial_message = None

    def restart(self, initial_message: str):
        self._restart(initial_message=initial_message, relevant=True)

    def retry(self, initial_message: str):
        self._restart(initial_message=initial_message, relevant=False)

    def _restart(self, *, initial_message: str, relevant: bool):
        self.reset()
        self._initial_message = initial_message
        raise RestartIterationSignal(relevant=relevant)

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self._finished == other._finished
            and self._context == other._context
            and self._expected_question_key == other._expected_question_key
            and self._current_state.__name__ == other._current_state.__name__
            and self._initial_message == other._initial_message
        )

    @property
    def finished(self) -> bool:
        return self._finished

    @abstractmethod
    def start(self, initial_message: str):
        pass

    def say(self, message: str):
        self._have_new_message(message=message)

    def _have_new_message(self, *, message: str):
        if message in self._context:
            return
        self._context[message] = None
        self._answers.append(message)

    def ask(self, question: str, direct_to: Optional[Callable] = None) -> str:
        return self._need_new_message(question=question, direct_to=direct_to, relevant=True)

    def specify(self, question: str, direct_to: Optional[Callable] = None):
        return self._need_new_message(question=question, direct_to=direct_to, relevant=False)

    def _need_new_message(self, *, question: str, direct_to: Optional[Callable], relevant: bool) -> str:
        self._have_new_message(message=question)
        if direct_to:
            self._context = {}
            self._current_state = direct_to
            self._initial_message = None
        else:
            if question in self._context:
                answer = self._context[question]
                if answer is not None:
                    return answer
            self._expected_question_key = question
        raise FinishIterationSignal(relevant=relevant)

    def _exit(self, *, message: str, relevant: bool):
        self._have_new_message(message=message)
        self.reset()
        self._finished = True
        self._current_state = None
        raise FinishIterationSignal(relevant=relevant)

    def finish(self, message: str):
        self._exit(message=message, relevant=True)

    def abort(self, reason: str):
        self._exit(message=reason, relevant=False)

    def send(self, message: str):
        if self.finished:
            raise StopIteration

        if self._expected_question_key:
            self._context[self._expected_question_key] = message
            self._expected_question_key = None

        self._answers = []
        relevant = True

        self._initial_message = self._initial_message or message

        while True:
            try:
                self._current_state(self._initial_message)
                self._finished = True
                break
            except IterationSignal as signal:
                relevant &= signal.relevant
                if isinstance(signal, FinishIterationSignal):
                    break

        return SkillResult(answers=self._answers, relevant=relevant)



from typing import Callable, List, Optional

from millet.context import AgentContext, RAMAgentContext
from millet.skill import Skill


class Conversation:
    def __init__(self, agent: "Agent", user_id: str):
        self.agent = agent
        self.user_id = user_id

    def query(self, message: str) -> List[str]:
        return self.agent.query(message, self.user_id)


class Agent:
    def __init__(self, skill_classifier: Callable[[str], List[Skill]], context: Optional[AgentContext] = None):
        if not callable(skill_classifier):
            raise TypeError("skill_classifier must be a function")

        if not context:
            context = RAMAgentContext()

        self.__skill_classifier = skill_classifier
        self.context = context

    def conversation_with_user(self, user_id: str) -> Conversation:
        return Conversation(agent=self, user_id=user_id)

    def query(self, message: str, user_id: str) -> List[str]:
        user_context = self.context.get_user_context(user_id)

        if not user_context.skills:
            user_context.skills = self.__skill_classifier(message)

        skills = user_context.skills[:]

        answers = []

        i = 0
        while i < len(skills):
            skill = skills[i]
            skill.global_context = user_context.params
            skill_result = skill.send(message)
            if not skill_result.relevant:
                skills = self.__skill_classifier(message)
                if skills:
                    user_context.skills = skills
                    self.context.set_user_context(user_id, user_context)
                    return self.query(message, user_id)

            answers += skill_result.answers
            if not skill.finished:
                break

            user_context.skills.pop(0)
            i += 1

        self.context.set_user_context(user_id, user_context)

        return answers



from redis import Redis

from millet.context import UserContext, RedisAgentContext, PickleSerializer, RAMAgentContext


def test_default_user_context(bob_id: str, redis: Redis):
    default_user_context = UserContext(params={}, skills=[])

    agent_context = RedisAgentContext(redis=redis)
    user_context = agent_context.get_user_context(bob_id)

    assert user_context == default_user_context


def test_reload_ram_context(bob_id: str, user_context: UserContext, redis: Redis):
    default_user_context = UserContext(params={}, skills=[])

    agent_context = RAMAgentContext()
    agent_context.set_user_context(bob_id, user_context)

    new_agent_context = RAMAgentContext()
    loaded_user_context = new_agent_context.get_user_context(bob_id)

    assert loaded_user_context == default_user_context


def test_pickle_reserialize(user_context: UserContext):
    serializer = PickleSerializer()
    serialized_user_context = serializer.dumps(user_context)
    serializer = PickleSerializer()
    deserailized_user_context = serializer.loads(serialized_user_context)

    assert deserailized_user_context == user_context


def test_reload_redis_context(bob_id: str, user_context: UserContext, redis: Redis):
    agent_context = RedisAgentContext(redis=redis)
    agent_context.set_user_context(bob_id, user_context)

    new_agent_context = RedisAgentContext(redis=redis)
    loaded_user_context = new_agent_context.get_user_context(bob_id)

    assert loaded_user_context.params == user_context.params



from typing import List, Type

import pytest
from redis import Redis

from millet import Agent, Skill
from millet.context import RAMAgentContext, RedisAgentContext


def test_answer_agent(bob_id: str):
    class EchoSkill(Skill):
        def start(self, initial_message: str):
            self.say(initial_message)

    def skill_classifier(message: str) -> List[Skill]:
        return [EchoSkill()]

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["Hello"]


def test_incorrect_skill_classifier():
    with pytest.raises(TypeError):
        Agent(skill_classifier=None)


def test_choice_of_skills(bob_id: str):
    class GreetingSkill(Skill):
        def start(self, initial_message: str):
            self.say("Hi")

    class PartingSkill(Skill):
        def start(self, initial_message: str):
            self.say("Bye")

    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "Hello" in message:
            skills.append(GreetingSkill())

        if "Goodbye" in message:
            skills.append(PartingSkill())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["Hi"]
    assert conversation.query("Goodbye") == ["Bye"]
    assert conversation.query("How are you?") == []


def test_choice_of_many_skills(bob_id: str):
    class GreetingSkill(Skill):
        def start(self, initial_message: str):
            self.say("Hello")

    class IntroduceYourselfSkill(Skill):
        def start(self, initial_message: str):
            self.say("It's me")

    def skill_classifier(message: str) -> List[Skill]:
        skills = [GreetingSkill(), IntroduceYourselfSkill()]
        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["Hello", "It's me"]


def test_separation_context_on_users(bob_id: str, alice_id: str, meeting_agent: Agent):
    conversation_with_bob = meeting_agent.conversation_with_user(bob_id)
    conversation_with_alice = meeting_agent.conversation_with_user(alice_id)

    assert conversation_with_bob.query("Hello") == ["What is your name?"]
    assert conversation_with_alice.query("Hello") == ["What is your name?"]

    assert conversation_with_bob.query("Bob") == ["Nice to meet you Bob!"]
    assert conversation_with_alice.query("Alice") == ["Nice to meet you Alice!"]


def test_query_without_conversation(bob_id: str, meeting_agent: Agent):
    assert meeting_agent.query("Hello", bob_id) == ["What is your name?"]
    assert meeting_agent.query("Bob", bob_id) == ["Nice to meet you Bob!"]


def test_multi_answers(bob_id: str):
    class MoodSkill(Skill):
        def start(self, initial_message: str):
            self.say("Hello")
            self.say("Good day!")
            self.ask("How are you?", direct_to=self.waiting_mood)

        def waiting_mood(self, mood: str):
            self.say("Goodbye")

    def skill_classifier(message: str) -> List[Skill]:
        return [MoodSkill()]

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["Hello", "Good day!", "How are you?"]


def test_continuous_conversation(bob_id: str, meeting_agent: Agent):
    conversation = meeting_agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["What is your name?"]
    assert conversation.query("Bob") == ["Nice to meet you Bob!"]


def test_ram_persistent_continuous_conversation(bob_id: str, meeting_skill_class: Agent):
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "Hello" in message:
            skills.append(meeting_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier, context=RAMAgentContext())
    conversation = agent.conversation_with_user(bob_id)
    assert conversation.query("Hello") == ["What is your name?"]

    agent = Agent(skill_classifier=skill_classifier, context=RAMAgentContext())
    conversation = agent.conversation_with_user(bob_id)
    assert conversation.query("Bob") == []


def test_redis_persistent_continuous_conversation(bob_id: str, redis: Redis, age_skill_class: Type[Skill]):
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "age" in message:
            skills.append(age_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier, context=RedisAgentContext(redis=redis))
    conversation = agent.conversation_with_user(bob_id)
    assert conversation.query("What about age?") == ["How old are you?"]

    agent = Agent(skill_classifier=skill_classifier, context=RedisAgentContext(redis=redis))
    conversation = agent.conversation_with_user(bob_id)
    assert conversation.query("42") == ["Ok"]


def test_default_context():
    agent = Agent(skill_classifier=lambda message: [])

    assert isinstance(agent.context, RAMAgentContext)


def test_global_context_change_in_skill(bob_id: str):
    class AgeSkill(Skill):
        def start(self, initial_message: str):
            age = self.global_context.get("age")
            if not age:
                self.ask("How old are you?", direct_to=self.waiting_age)
            self.say(f"You are {age} years old")

        def waiting_age(self, age: str):
            self.global_context["age"] = age
            self.say(f"You are {age} years old")

    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "age" in message:
            skills.append(AgeSkill())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("What about age?") == ["How old are you?"]
    assert conversation.query("42") == ["You are 42 years old"]
    assert conversation.query("What about age") == ["You are 42 years old"]


def test_specify(bob_id: str, age_skill_class: Type[Skill]):
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "age" in message:
            skills.append(age_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("What about age?") == ["How old are you?"]
    assert conversation.query("Forty two") == ["Incorrect age: expected number, repeat pls"]
    assert conversation.query("42") == ["Ok"]


def test_move_to_new_skill_when_specify(bob_id: str, age_skill_class: Type[Skill], meeting_skill_class: Type[Skill]):
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "age" in message:
            skills.append(age_skill_class())

        if "Hello" in message:
            skills.append(meeting_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("What about age?") == ["How old are you?"]
    assert conversation.query("Hello") == ["What is your name?"]


def test_do_not_move_to_new_skill_when_not_specify(bob_id: str, age_skill_class: Type[Skill], meeting_skill_class: Type[Skill]):
    def skill_classifier(message: str) -> List[Skill]:
        skills = []

        if "age" in message:
            skills.append(age_skill_class())

        if "Hello" in message:
            skills.append(meeting_skill_class())

        return skills

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["What is your name?"]
    assert conversation.query("What about age?") == ["Nice to meet you What about age?!"]


def test_inline_ask(bob_id: str):
    class MeetingSkill(Skill):
        def start(self, initial_message: str):
            self.say(f"Hello")
            name = self.ask(question="What is your name?")
            self.say(f"Nice to meet you {name}!")

    def skill_classifier(message: str) -> List[Skill]:
        return [MeetingSkill()]

    agent = Agent(skill_classifier=skill_classifier)
    conversation = agent.conversation_with_user(bob_id)

    assert conversation.query("Hello") == ["Hello", "What is your name?"]
    assert conversation.query("Bob") == ["Nice to meet you Bob!"]



import pickle
from abc import ABC, abstractmethod
from typing import List, Any

from redis import Redis

from millet.skill import Skill


class UserContext:
    def __init__(self, params: dict, skills: List[Skill]):
        self.params = params
        self.skills = skills

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.params == other.params and self.skills == other.skills


class AgentContext(ABC):
    @property
    def empty_user_context(self) -> UserContext:
        return UserContext(params={}, skills=[])

    @abstractmethod
    def set_user_context(self, user_id: str, user_context: UserContext) -> None:
        pass

    @abstractmethod
    def get_user_context(self, user_id: str) -> UserContext:
        pass


class RAMAgentContext(AgentContext):
    def __init__(self):
        self.__storage = dict()

    def set_user_context(self, user_id: str, user_context: UserContext) -> None:
        self.__storage[user_id] = user_context

    def get_user_context(self, user_id: str) -> UserContext:
        return self.__storage.get(user_id, self.empty_user_context)


class PickleSerializer:
    def loads(self, serialized_param: str) -> Any:
        param = pickle.loads(serialized_param.encode())
        return param

    def dumps(self, param: Any) -> str:
        serialized_param = pickle.dumps(param, 0).decode()
        return serialized_param


class RedisAgentContext(AgentContext):
    def __init__(self, redis: Redis):
        self.redis = redis
        self.serializer = PickleSerializer()

    def __serialize_user_context(self, user_context: UserContext) -> str:
        return self.serializer.dumps(user_context)

    def __deserialize_user_context(self, serialized_user_context: str) -> UserContext:
        return self.serializer.loads(serialized_user_context)

    def set_user_context(self, user_id: str, user_context: UserContext) -> None:
        serialized_user_context = self.__serialize_user_context(user_context)
        self.redis.set(user_id, serialized_user_context)

    def get_user_context(self, user_id: str) -> UserContext:
        serialized_user_context = self.redis.get(user_id)
        if not serialized_user_context:
            return self.empty_user_context

        serialized_user_context = serialized_user_context.decode()
        user_context = self.__deserialize_user_context(serialized_user_context)
        return user_context



import io
import re

from setuptools import find_packages
from setuptools import setup

with io.open("README.rst", "rt", encoding="utf8") as f:
    readme = f.read()

with io.open("src/millet/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name="Millet",
    version=version,
    project_urls={
        "Code": "https://github.com/odryfox/millet",
        "Issue tracker": "https://github.com/odryfox/millet/issues",
    },
    license="BSD-3-Clause",
    author="Vyacheslav Rusov",
    author_email="odryfox@gmail.com",
    description="A simple framework for building complex dialogue systems.",
    long_description=readme,
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    extras_require={
        "dev": [
            "pytest",
            "sphinx",
            "redis",
        ],
        "docs": [
            "sphinx",
        ],
    },
)
